#ifndef _TRANSKODELISTVIEW_H_
#define _TRANSKODELISTVIEW_H_

/***************************************************************************
*   Copyright (C) 2006 by Sergio Pistone                                  *
*   sergio_pistone@yahoo.com.ar                                           *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <transkodeconfig.h>
#include <transkodefileworker.h>

#include <klistview.h>

class TransKodeListViewItem;

class TransKodeListView : public KListView
{
	Q_OBJECT

	friend class TransKodeListViewItem;

	public:

		enum Columns
		{
			c_src_path = 0, c_src_dir = 1, c_src_file = 2,
			c_dst_path = 3, c_dst_dir = 4, c_dst_file = 5,
			c_profile = 6, c_status_msg = 7, c_queue = 8, SIZE
		};

		TransKodeListView( QWidget *parent, const TransKodeConfig& config, TransKodeFileWorker& worker );

		TransKodeListViewItem* addFile( const QString& file, TransKodeListViewItem* after = 0 );
		TransKodeListViewItem* addFiles( const QStringList& files, TransKodeListViewItem* after = 0 );

		void saveConfig();
		void loadConfig();

		virtual bool eventFilter( QObject* object, QEvent* event );

		void setItemHorizontalMargin( uint margin );
		uint itemHorizontalMargin();

	signals:
		void message( const QString& msg );

		void jobFinished();
		void finishedCleared();

		void allCleared();
		void contentAdded();

	public slots:
		void addFiles();
		void removeFiles();

		void queueFiles();
		void dequeueFiles();

		void selectAll();
		void refresh();
		void clearAll();
		void clearFinished();

	protected:
		bool acceptDrag( QDropEvent* event ) const;
		QDragObject* dragObject();

		void filesInDir( const QString& dir, QStringList& files );
		virtual void contextMenuEvent( QContextMenuEvent* e );

	protected slots:

		void onDropEvent( QDropEvent* event, QListViewItem* after );

		void slotCurrentChanged( QListViewItem* current );
		void slotSetSortColumn( int column );
		void slotAdjustColumnWidth( int column );
		void slotRepaint();

		void adjustGeometry();

	protected:
		const TransKodeConfig& m_config;
		TransKodeFileWorker& m_worker;
		QString m_lastDir;
		QValueList<int> m_sortColumns;

		uint m_horizMargin;
};

#endif
