/***************************************************************************
 *   Copyright (C) 2006 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "transkodelistviewitem.h"
#include "transkodelistview.h"

#include <qpainter.h>
#include <qheader.h>

#define MAX_INT 2147483647
#define MIN_INT -2147483647

TransKodeListViewItem::TransKodeListViewItem( QListView* parent, const TransKodeConfig& config, const QString& srcPath ):
	QObject( 0, "TransKodeListViewItem" ),
	KListViewItem( parent ),
	m_jobError( false ),
	m_delete( false ),
	m_progressBar( new QProgressBar( 100, listView()->viewport() ) )
{
	QFileInfo fileInfo( srcPath );
	setText( TransKodeListView::c_src_path, fileInfo.filePath() );
	setText( TransKodeListView::c_src_dir, fileInfo.dirPath() );
	setText( TransKodeListView::c_src_file, fileInfo.fileName() );

	m_status = waitingprofile;
	setText( TransKodeListView::c_status_msg, i18n( "waiting profile" ) );

	m_job = new TransKodeFileJob( config, srcPath );

	connect( m_job, SIGNAL( mtProfileErrorSet(long,const QString&) ), this, SLOT( setProfileError(long) ) );
	connect( m_job, SIGNAL( mtProfileErrorUnset(long,const QString&) ), this, SLOT( unsetProfileError(long) ) );
	connect( m_job, SIGNAL( mtDstPathChanged(long,const QString&) ), this, SLOT( setDstPath(long,const QString&) ) );

	connect( m_job, SIGNAL( mtEnqueued(long,int) ), this, SLOT( setEnqueued(long,int) ) );
	connect( m_job, SIGNAL( mtPosChanged(long,int) ), this, SLOT( setQueuePos(long,int) ) );
	connect( m_job, SIGNAL( mtCancelled(long) ), this, SLOT( setCancelled(long) ) );

	connect( m_job, SIGNAL( mtStarted(long,const QString&,const QString&,int) ), this, SLOT( setStarted(long) ) );
	connect( m_job, SIGNAL( mtFinished(long,const QString&,const QString&,int) ), this, SLOT( setFinished(long) ) );
	connect( m_job, SIGNAL( mtError(long,const QString&,const QString&,int) ), this, SLOT( setJobError(long,const QString&,const QString&) ) );
	connect( m_job, SIGNAL( mtProgress(long,int,int) ), this, SLOT( setProgress(long,int) ) );
}

TransKodeListViewItem::TransKodeListViewItem( QListViewItem* parent, const TransKodeConfig& config, const QString& srcPath ):
	QObject( 0, "TransKodeListViewItem" ),
	KListViewItem( parent ),
	m_jobError( false ),
	m_delete( false ),
	m_progressBar( new QProgressBar( 100, listView()->viewport() ) )
{
	QFileInfo fileInfo( srcPath );
	setText( TransKodeListView::c_src_path, fileInfo.filePath() );
	setText( TransKodeListView::c_src_dir, fileInfo.dirPath() );
	setText( TransKodeListView::c_src_file, fileInfo.fileName() );

	m_status = waitingprofile;
	setText( TransKodeListView::c_status_msg, i18n( "waiting profile" ) );

	m_job = new TransKodeFileJob( config, srcPath );

	connect( m_job, SIGNAL( mtProfileErrorSet(long,const QString&) ), this, SLOT( setProfileError(long) ) );
	connect( m_job, SIGNAL( mtProfileErrorUnset(long,const QString&) ), this, SLOT( unsetProfileError(long) ) );
	connect( m_job, SIGNAL( mtDstPathChanged(long,const QString&) ), this, SLOT( setDstPath(long,const QString&) ) );

	connect( m_job, SIGNAL( mtEnqueued(long,int) ), this, SLOT( setEnqueued(long,int) ) );
	connect( m_job, SIGNAL( mtPosChanged(long,int) ), this, SLOT( setQueuePos(long,int) ) );
	connect( m_job, SIGNAL( mtCancelled(long) ), this, SLOT( setCancelled(long) ) );

	connect( m_job, SIGNAL( mtStarted(long,const QString&,const QString&,int) ), this, SLOT( setStarted(long) ) );
	connect( m_job, SIGNAL( mtFinished(long,const QString&,const QString&,int) ), this, SLOT( setFinished(long) ) );
	connect( m_job, SIGNAL( mtError(long,const QString&,const QString&,int) ), this, SLOT( setJobError(long,const QString&) ) );
	connect( m_job, SIGNAL( mtProgress(long,int,int) ), this, SLOT( setProgress(long,int) ) );
}

TransKodeListViewItem::~TransKodeListViewItem()
{
	m_progressBar->hide();
	delete m_job;
}

void TransKodeListViewItem::refresh()
{
	if ( isDone() )
		m_progressBar->hide();

	m_jobError = false;
// 	m_status = profileset;
// 	setText( TransKodeListView::c_status_msg, i18n( "profile set" ) );
//
// 	setText( TransKodeListView::c_profile, profile );

	m_job->refresh();
}

void TransKodeListViewItem::setProfile( const QString& profile )
{
	if ( isDone() )
		m_progressBar->hide();

	m_jobError = false;
	m_status = profileset;
	setText( TransKodeListView::c_status_msg, i18n( "profile set" ) );

	setText( TransKodeListView::c_profile, profile );

	m_job->setProfile( profile );
}


void TransKodeListViewItem::enqueue( TransKodeFileWorker* worker )
{
	if ( m_job->worker() == 0 && m_status == profileset && !m_job->profileError() )
		m_job->enqueue( worker );
}

void TransKodeListViewItem::cancel()
{
	if ( m_job->worker() != 0 )
		m_job->tryCancel();
}

void TransKodeListViewItem::remove()
{
	if ( m_job->worker() != 0 )
	{
		m_delete = true;
		m_job->tryCancel();
	}
	else
		delete this;
}

bool TransKodeListViewItem::isDone() const
{
	return m_job->progress() == 100;
}

bool TransKodeListViewItem::isQueueable() const
{
	return !m_job->enqueued() && !m_job->profileError();
}

bool TransKodeListViewItem::isRefresheable() const
{
	return !m_job->enqueued() && m_status != waitingprofile;
}

bool TransKodeListViewItem::isEnqueued() const
{
	return m_job->enqueued();
}

const TransKodeFileJob* TransKodeListViewItem::job() const
{
	return m_job;
}

void TransKodeListViewItem::setDstPath( long, const QString& dstPath )
{
	if ( dstPath.isEmpty() )
	{
		setText( TransKodeListView::c_dst_path, "" );
		setText( TransKodeListView::c_dst_dir, "" );
		setText( TransKodeListView::c_dst_file, "" );
	}
	else
	{
		QFileInfo fileInfo( dstPath );
		setText( TransKodeListView::c_dst_path, fileInfo.filePath() );
		setText( TransKodeListView::c_dst_dir, fileInfo.dirPath() );
		setText( TransKodeListView::c_dst_file, fileInfo.fileName() );
	}
}

void TransKodeListViewItem::setEnqueued( long, int pos )
{
	m_status = queued;

	m_progressBar->hide();

	setText( TransKodeListView::c_status_msg, i18n( "queued" ) );
	setText( TransKodeListView::c_queue, QString::number( pos ) );
}

void TransKodeListViewItem::setQueuePos( long, int pos )
{
	setText( TransKodeListView::c_queue, QString::number( pos ) );
}

void TransKodeListViewItem::setProfileError( long )
{
	setText( TransKodeListView::c_status_msg, i18n( "profile error" ) );
}

void TransKodeListViewItem::unsetProfileError( long )
{
	setText( TransKodeListView::c_status_msg, i18n( "profile set" ) );
}

void TransKodeListViewItem::setCancelled( long )
{
	if ( !m_delete )
	{
		m_status = profileset;
		setText( TransKodeListView::c_queue, "" );
		setText( TransKodeListView::c_status_msg, i18n( "profile set" ) );
		m_progressBar->hide();
	}
	else
		delete this;
}

void TransKodeListViewItem::setStarted( long )
{
	m_status = transcoding;
	setText( TransKodeListView::c_queue, "" );

	m_progressBar->setProgress( 0 );
	adjustGeometry();
	//m_progressBar->show();
	repaint();
}

void TransKodeListViewItem::setProgress( long, int progress )
{
	if ( progress == 0 )
		setStarted( -1 );
	else if ( progress == 100 )
		setFinished( -1 );
	else
	{
		m_progressBar->setProgress( progress );
		repaint();
	}
}


void TransKodeListViewItem::setFinished( long )
{
	m_status = profileset;

	m_progressBar->setProgress( 100 );
	repaint();
}

void TransKodeListViewItem::setJobError( long, const QString&, const QString& errorMsg )
{
	m_status = profileset;
	m_jobError = true;

	setText( TransKodeListView::c_status_msg, errorMsg );

	m_progressBar->hide(); // to show error message instead of progress bar
}


int TransKodeListViewItem::compare( QListViewItem* item, int, bool ascending ) const
{
	TransKodeListView* list = (TransKodeListView*)listView();

	int ret = 0;
	for ( unsigned idx = 0; idx < list->m_sortColumns.size(); ++idx )
		if ( (ret = compareAux( item, idx, ascending )) )
			break;

	return ret;
}


int TransKodeListViewItem::compareAux( QListViewItem* item, int column, bool ascending ) const
{
	if ( item->rtti() != TRANSKODE )
		return KListViewItem::compare( item, column, ascending );

	if ( column == TransKodeListView::c_queue )
	{
		QString current = text( column );
		QString next = item->text( column );;

		bool ok;
		int c = current.toInt( &ok ); if ( !ok ) c = column==TransKodeListView::c_queue ? MAX_INT : -1;
		int n = next.toInt( &ok ); if ( !ok ) n = column==TransKodeListView::c_queue ? MAX_INT : -1;;

		return c-n;
	}
	else if ( column == TransKodeListView::c_status_msg )
	{
		if ( m_job->progress() >= 0 )
		{
			if ( ((TransKodeListViewItem*)item)->m_job->progress() >= 0 )
				return m_job->progress() - ((TransKodeListViewItem*)item)->m_job->progress();
			else
				return 1;
		}
		else if ( ((TransKodeListViewItem*)item)->m_job->progress() >= 0 )
			return -1;
	}

	return KListViewItem::compare( item, column, ascending );
}

// reimplemented to draw progress bar
void TransKodeListViewItem::paintCell( QPainter* p, const QColorGroup &cg, int column, int width, int align )
{
	if ( column == TransKodeListView::c_status_msg && m_status == transcoding || isDone() )
		adjustGeometry();

	QColorGroup auxcg( cg );
	QColor c = auxcg.text();

	if ( ( m_jobError || ( m_job->profileError() && m_status != transcoding && m_status != waitingprofile ) ) )
		auxcg.setColor( QColorGroup::Text, Qt::red );
	else if ( isDone() )
		auxcg.setColor( QColorGroup::Text, Qt::gray );

	int h = height();
	int margin = ((TransKodeListView*)listView())->itemHorizontalMargin();
	int w = width - 2*margin;
	// clean background
	p->fillRect( 0, 0, width, h, isSelected() ? auxcg.highlight() : auxcg.base() );
	p->setPen( isSelected() ? auxcg.highlightedText() : auxcg.text() );

	if ( align & Qt::AlignCenter )
	{
		p->drawText(margin, 0, w, h, align|Qt::SingleLine|Qt::ExpandTabs, text( column ), -1 );
		return;
	}

	QRect textRect = p->boundingRect( margin, 0, w, h, align|Qt::SingleLine|Qt::ExpandTabs, text( column ) );
	if ( textRect.width() <= w )
		p->drawText(margin, 0, w, h, align|Qt::SingleLine|Qt::ExpandTabs, text( column ), -1 );
	else
	{
		textRect = p->boundingRect( margin, 0, w, h, Qt::AlignLeft, "... " );
		p->drawText( margin, 0, textRect.width(), h, Qt::AlignLeft|Qt::SingleLine|Qt::ExpandTabs, "...", -1 );
		p->drawText( margin+textRect.width(), 0, w-textRect.width(), h, Qt::AlignRight|Qt::SingleLine|Qt::ExpandTabs, text( column ), -1 );
	}

	//KListViewItem::paintCell( p, auxcg, column, width, align );
}

void TransKodeListViewItem::adjustGeometry()
{
	if ( m_job->progress() >= 0 )
	{
		QRect r = listView()->itemRect( this );
		QHeader* h = listView()->header();
		r.setLeft( h->sectionPos( TransKodeListView::c_status_msg ) - h->offset() );
		r.setWidth( h->sectionSize( TransKodeListView::c_status_msg ) );
		if ( r.isValid() )
		{
			m_progressBar->setGeometry( r );
			m_progressBar->show();
		}
	}
}

#include "transkodelistviewitem.moc"
