#ifndef _TRANSKODELISTVIEWITEM_H_
#define _TRANSKODELISTVIEWITEM_H_

/***************************************************************************
*   Copyright (C) 2006 by Sergio Pistone                                  *
*   sergio_pistone@yahoo.com.ar                                           *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <transkodefileworker.h>

#include <qobject.h>
#include <qprogressbar.h>

#include <klistview.h>
#include <klocale.h>

// a random number to identify TransKodeListView items type
#define TRANSKODE 1353

class TransKodeListView;

class TransKodeListViewItem : public QObject, public KListViewItem
{
	Q_OBJECT

	public:
		typedef enum { waitingprofile = 0, profileset, queued, transcoding } TItemState;

		TransKodeListViewItem( QListView* parent, const TransKodeConfig& config, const QString& srcPath );
		TransKodeListViewItem( QListViewItem* parent, const TransKodeConfig& config, const QString& srcPath );

		virtual ~TransKodeListViewItem();

		void refresh();
		void setProfile( const QString& profile );
		void enqueue( TransKodeFileWorker* worker );
		void cancel();
		void remove();

		bool isEnqueued() const;
		bool isQueueable() const;
		bool isRefresheable() const;
		bool isDone() const;

		const TransKodeFileJob* job() const;

		void adjustGeometry();

	protected slots:

		void setDstPath( long jobID, const QString& path );
		void setEnqueued( long jobID, int queuePos );
		void setQueuePos( long jobID, int queuePos );
		void setProfileError( long jobID );
		void unsetProfileError( long jobID );
		void setCancelled( long jobID );
		void setStarted( long jobID );
		void setProgress( long jobID, int progress );
		void setFinished( long jobID );
		void setJobError( long jobID, const QString& srcPath, const QString& errorMsg );


	protected:
		static TransKodeListViewItem* getItem( TransKodeListView* tView, int ID );

		// reimplemented to provide runtime type information
		inline int rtti() const { return TRANSKODE; }

		// reimplemented to sort based on more than one column
		virtual int compare( QListViewItem* item, int column, bool ascending ) const;
		virtual int compareAux( QListViewItem* item, int column, bool ascending ) const;

		// reimplemented to draw progress bar
		virtual void paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int align );

	private:

		TItemState m_status;
		bool m_jobError;
		bool m_delete;

		TransKodeFileJob* m_job;

		QProgressBar* m_progressBar;
};

#endif
