/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "commondefs.h"

#include <qthread.h>
#include <qmutex.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qdeepcopy.h>
#include <qdatastream.h>

#ifndef Q_WS_WIN
	#include <unistd.h>
#endif

/**
 * GeneralOpt
*/

QString GeneralOpt::toQString( GeneralOpt::TGeneralOpt option )
{
	switch ( option )
	{
		case GeneralOpt::naming_scheme:		return "naming_scheme";
		case GeneralOpt::notify:			return "notify";
		case GeneralOpt::confirm:			return "confirm";
		case GeneralOpt::overwrite:			return "overwrite";
		case GeneralOpt::case_insensitive:	return "case_insensitive";
		case GeneralOpt::replace_chars:		return "replace_chars";
		case GeneralOpt::rip_duration:		return "rip_duration";
		case GeneralOpt::rip_incomplete:	return "rip_incomplete";
		case GeneralOpt::proc_priority:		return "proc_priority";
		case GeneralOpt::show_tray:			return "show_tray";
		case GeneralOpt::log:				return "log";
		case GeneralOpt::temp_dir:			return "temp_dir";
		default:							return "Unknown";
	}
}

GeneralOpt::TGeneralOpt GeneralOpt::fromQString( const QString& option )
{
	if	( option == "naming_scheme" )			return GeneralOpt::naming_scheme;
	else if	( option == "notify" )				return GeneralOpt::notify;
	else if	( option == "confirm" )				return GeneralOpt::confirm;
	else if	( option == "overwrite" )			return GeneralOpt::overwrite;
	else if	( option == "case_insensitive" ) 	return GeneralOpt::case_insensitive;
	else if	( option == "replace_chars" )		return GeneralOpt::replace_chars;
	else if	( option == "rip_duration" )		return GeneralOpt::rip_duration;
	else if	( option == "rip_incomplete" )		return GeneralOpt::rip_incomplete;
	else if	( option == "proc_priority" )		return GeneralOpt::proc_priority;
	else if	( option == "show_tray" )			return GeneralOpt::show_tray;
	else if	( option == "log" )					return GeneralOpt::log;
	else if	( option == "temp_dir" )			return GeneralOpt::temp_dir;
	else										return GeneralOpt::UNKNOWN;
}


/**
 * ProfileOpt
*/

QString ProfileOpt::toQString( ProfileOpt::TProfileOpt option )
{
	switch ( option )
	{
		case ProfileOpt::naming_scheme:	return "naming_scheme";
		case ProfileOpt::encode_type:	return "encode_type";
		case ProfileOpt::extension:		return "extension";
		case ProfileOpt::switches:		return "switches";
		case ProfileOpt::show:			return "show";
		case ProfileOpt::bypass:		return "bypass";
		default:						return "Unknown";
	}
}

ProfileOpt::TProfileOpt ProfileOpt::fromQString( const QString& option )
{
	if 		( option == "naming_scheme" )	return ProfileOpt::naming_scheme;
	else if	( option == "encode_type" )		return ProfileOpt::encode_type;
	else if	( option == "extension" )		return ProfileOpt::extension;
	else if	( option == "switches" )		return ProfileOpt::switches;
	else if	( option == "show" )			return ProfileOpt::show;
	else if ( option == "bypass" )			return ProfileOpt::bypass;
	else									return ProfileOpt::UNKNOWN;
}


/**
 * CaseFunc
*/

QString CaseFunc::toQString( CaseFunc::TCaseFunc caseFunction )
{
	switch ( caseFunction )
	{
		case CaseFunc::titlecase:	return "titlecase";
		case CaseFunc::capitalize:	return "capitalize";
		case CaseFunc::lowercase:	return "lowercase";
		case CaseFunc::uppercase:	return "uppercase";
		case CaseFunc::leaveasis:	return "";
		default:					return "Unknown";
	}
}

CaseFunc::TCaseFunc CaseFunc::fromQString( const QString& caseFunction )
{
	if 		( caseFunction == "titlecase" )		return CaseFunc::titlecase;
	else if	( caseFunction == "capitalize" )	return CaseFunc::capitalize;
	else if	( caseFunction == "lowercase" )		return CaseFunc::lowercase;
	else if	( caseFunction == "uppercase" )		return CaseFunc::uppercase;
	else if	( caseFunction == "" )				return CaseFunc::leaveasis;
	else										return CaseFunc::UNKNOWN;
}


/**
 * Tag
 */

QString Tag::toQString( Tag::TTag tag )
{
	switch ( tag )
	{
		case Tag::artist:	return "artist";
		case Tag::album:	return "album";
		case Tag::year:		return "year";
		case Tag::track:	return "track";
		case Tag::title:	return "title";
		case Tag::genre:	return "genre";
		case Tag::composer:	return "composer";
		case Tag::discnum:	return "discnum";
		case Tag::comment:	return "comment";
		default:			return "Unknown";
	}
}

Tag::TTag Tag::fromQString( const QString& tag )
{
	if 		( tag == "artist" )		return Tag::artist;
	else if	( tag == "album" )		return Tag::album;
	else if	( tag == "year" )		return Tag::year;
	else if	( tag == "track" )		return Tag::track;
	else if	( tag == "title" )		return Tag::title;
	else if	( tag == "genre" )		return Tag::genre;
	else if	( tag == "comment" )	return Tag::comment;
	else if	( tag == "composer" )	return Tag::composer;
	else if	( tag == "discnum" )	return Tag::discnum;
	else							return Tag::UNKNOWN;
}

/**
 * String
 */

QString String::title( const QString& text )
{
	QString auxText = text.lower();
	const QString separators = " -_([:,;./\\\t\n\"";
	int len = strlen( separators ), sepIdx = 0;
	while ( sepIdx < len )
	{
		QStringList auxWordsList;
		QStringList wordsList = QStringList::split( separators[sepIdx], auxText, true );
		for ( QStringList::Iterator it = wordsList.begin(); it != wordsList.end(); ++it )
			if ( ! (*it).isEmpty() )
				auxWordsList.append( (*it)[0].upper()+(*it).mid(1) );
			else
				auxWordsList.append( "" );
		auxText = auxWordsList.join( separators[sepIdx] );
		sepIdx++;
	}
	return auxText;
}

QString String::capitalize( const QString& text )
{
	return text.isEmpty() ? "" : text[0].upper() + text.mid(1).lower();
}

int String::rfindFunctionStart( const QString& text )
{
	int pos[4];
	pos[0] = text.findRev( "capitalize{" );
	pos[1] = text.findRev( "titlecase{" );
	pos[2] = text.findRev( "uppercase{" );
	pos[3] = text.findRev( "lowercase{" );

	return QMAX( QMAX( QMAX( pos[0], pos[1] ), pos[2] ), pos[3] );
}

int String::rfindFunctionEnd( const QString& text, int startPos )
{
	int pos = startPos, len = text.length();
	while ( pos < len )
	{
		// next is safe because idx will always be greater than 0
		if ( text[pos] == '}' && text[pos-1] != '\\' )
			return pos;
		pos++;
	}
	return -1;
}

bool String::isValidCaseExpression( const QString& expression )
{
	if ( expression.isEmpty() )
		return false;

    QString expr( expression );

	expr.replace( "\\{", "" );
	expr.replace( "\\}", "" );

	int startPos, endPos;
	while ( (startPos = String::rfindFunctionStart( expr )) != -1 )
	{
		endPos = String::rfindFunctionEnd( expr, startPos );
		if ( endPos == -1 )
			return false;
		QString funcParam = expr.mid( startPos, endPos - startPos );
		if ( funcParam.contains( '{' ) != 1 || funcParam.contains( '}' ) )
			return false;
		expr = expr.left( startPos ) + expr.mid( endPos + 1 );
	}

	return ( expr.contains( '{' ) || expr.contains( '}' ) ) ? false : true;
}

QString String::evaluateCaseExpression( const QString& expression )
{
	int startPos, endPos;
	QString expr = expression;

	while ( (startPos = rfindFunctionStart( expr )) != -1 )
	{
		endPos = rfindFunctionEnd( expr, startPos );
		QString transformed = "";
		if ( expr[startPos] == 't' )
			transformed = title( expr.mid( startPos + strlen( "titlecase{" ), endPos - startPos - strlen( "titlecase{" ) ) );
		else if ( expr[startPos] == 'c' )
			transformed = capitalize( expr.mid( startPos + strlen( "capitalize{" ), endPos - startPos - strlen( "capitalize{" ) ) );
		else if ( expr[startPos] == 'l' )
			transformed = expr.mid( startPos + strlen( "lowercase{" ), endPos - startPos - strlen( "lowercase{" ) ).lower();
		else if ( expr[startPos] == 'u' )
			transformed = expr.mid( startPos + strlen( "uppercase{" ), endPos - startPos - strlen( "uppercase{" ) ).upper();

		expr = expr.left( startPos ) + transformed + expr.mid( endPos + 1 );
	}

	return expr;
}

QString String::evaluateCase( const QString& text, const QString& caseFunction )
{
	return caseFunction.isEmpty() ? text : evaluateCaseExpression( caseFunction + "{" + text + "}" );
}

QString String::evaluateCase( const QString& text, CaseFunc::TCaseFunc caseFunction )
{
	return caseFunction == CaseFunc::leaveasis ?
		text : evaluateCaseExpression( CaseFunc::toQString( caseFunction ) + "{" + text + "}" );

}

QString String::quoteForShell( const QString& text )
{
	return "\"" + QString( text ).replace( "\"","\\\"" ).replace( "`","\\`" ) + "\"";
}

QString String::padNumber( const QString& string, unsigned width )
{
	if ( string.length() >= width )
		return string;

	QString paddedNumber;
	bool ok;
	int number = string.toUInt( &ok, 10 );
	if ( ok )
		return paddedNumber.sprintf( "%0" + QString::number( width ) + "d", number );
	else
		return string;
}

QString String::deepCopy( const QString& string )
{
	return QDeepCopy<QString>( string );
}

QStringList String::deepCopy( const QStringList& stringList )
{
	return QDeepCopy<QStringList>( stringList );
}


/**
 * System:
 */

QStringList System::s_potentialFiles;
QMutex System::s_potentialFilesMutex;

void System::addPotentialFile( const QString& path )
{
	if ( ! path.isEmpty() )
	{
		s_potentialFilesMutex.lock();

		s_potentialFiles.append( path );

		s_potentialFilesMutex.unlock();
	}
}

void System::delPotentialFile( const QString& path )
{
	if ( ! path.isEmpty() )
	{
		s_potentialFilesMutex.lock();

		s_potentialFiles.remove( path );

		s_potentialFilesMutex.unlock();
	}
}

void System::clearPotentialFiles()
{
	s_potentialFilesMutex.lock();

	s_potentialFiles.clear();

	s_potentialFilesMutex.unlock();
}

QString System::isPotentialFile( const QString& path, bool VFAT )
{
	s_potentialFilesMutex.lock();

	QString ret;
	if ( VFAT )
	{
		ret = "";
		for ( unsigned idx = 0; idx < s_potentialFiles.size(); ++idx )
			if ( path.lower() == s_potentialFiles[idx].lower() )
			{
				ret = s_potentialFiles[idx];
				break;
			}
	}
	else
		ret = s_potentialFiles.contains( path ) ? path : "";

	s_potentialFilesMutex.unlock();

	return ret;
}

// If necesary, changes the part of a given path to match the case of the existing one
QString System::correctVFATPath( const QString& path )
{
	bool tokenFound = true;
	QDir parcialPath( "/" );
	QStringList pathTokens = QStringList::split( "/", path );
	for ( QStringList::Iterator tokenIt = pathTokens.begin(); tokenIt != pathTokens.end(); ++tokenIt )
	{
		if ( tokenFound )
		{
			if ( !QFileInfo( parcialPath.path() + "/" + *tokenIt ).exists() )
			{
				LOG( "correctVFATPath: '" + *tokenIt + "' not found in '" + parcialPath.path() + "'", SYSTEM );
				LOG( "correctVFATPath: performing a case insensitive search", SYSTEM );

				QString lowerCaseToken = (*tokenIt).lower();
				QStringList dirContents = parcialPath.entryList( "*", QDir::All|QDir::System );
				tokenFound = false;
				for ( QStringList::Iterator dirContentsIt = dirContents.begin(); dirContentsIt != dirContents.end(); ++dirContentsIt )
				{
					if ( (*dirContentsIt).lower() == lowerCaseToken )
					{
						LOG( "correctVFATPath: found '" + *dirContentsIt + "'", SYSTEM );
						*tokenIt = *dirContentsIt;
						tokenFound = true;
						break;
					}
				}
			}
		}
		parcialPath.setPath( parcialPath.path() + "/" + *tokenIt );
	}

	return parcialPath.path();
}

// Checks if the given path exists; returns the existing path if it does or else returns "".
// If VFAT is true, path will be searched in a case insensitive way but the returned value
// will have the actual case of the existing path.
QString System::conflictivePath( const QString& path, bool VFAT, bool potentialFiles, const QString& exception )
{
	QString collidingPath = "";

	if ( VFAT ) // case insensitive test
	{
		// check for collitions with potential files:
		if ( potentialFiles )
		{
			collidingPath = isPotentialFile( path, true );
			if ( ! collidingPath.isEmpty() ) // there was a potential file with 'path'
				return collidingPath.lower() == exception.lower() ? "" /*exception*/ : collidingPath /*no exception*/;
		}

		// check for collitions with real files:
		collidingPath = QFileInfo( collidingPath = correctVFATPath( path ) ).exists() ? collidingPath : "";
		if ( ! collidingPath.isEmpty() ) // there was a real file with 'path'
			return collidingPath;
	}
	else // case sensitive test
	{
		// check for collitions with potential files:
		if ( potentialFiles )
		{
			collidingPath = isPotentialFile( path, false );
			if ( ! collidingPath.isEmpty() ) // there was a potential file with 'path'
				return collidingPath == exception ? "" /*exception*/ : collidingPath /*no exception*/;
		}

		// check for collitions with real files:
		collidingPath = QFileInfo( path ).exists() ? path : "";
		if ( ! collidingPath.isEmpty() ) // there was a real file with 'path'
			return collidingPath;
	}

	return "";
}

// If the path exists, appends a suffix before the extension wich is incremented
// until the resulting path doesn't exists. If onlyDir is true the resulting
// path will not conflict with a dir but may conflict with a file.
QString System::nonConflictivePath( const QString& path, bool onlyDir, bool VFAT, bool potentialFiles, const QString& exception )
{
	if ( onlyDir ) // waste of time
		potentialFiles = false;

	QFileInfo fileInfo( VFAT ? correctVFATPath( path ) : path );
	QString extension = fileInfo.extension( false );
	if ( ! extension.isEmpty() )
		extension = "." + extension;
	QString dirAndName = fileInfo.filePath();
	dirAndName.truncate( dirAndName.length() - extension.length() );
	if ( dirAndName.right( 1 ) == "." )
		dirAndName.truncate( dirAndName.length() - 1 );

	unsigned suffix = 2;

	while ( !conflictivePath( fileInfo.filePath(), VFAT, potentialFiles, exception ).isEmpty() &&
			( !onlyDir || fileInfo.isDir() || fileInfo.isSymLink() ) )
		fileInfo.setFile( dirAndName + "_" + QString::number( suffix++ ) + extension );

	return fileInfo.filePath();
}

bool System::recursiveMakeDir( const QString& path, QStringList* createdDirsList, bool VFAT )
{
	if ( createdDirsList )
		createdDirsList->clear();

	QDir parcialPath( "/" );
	QStringList tokens = QStringList::split( "/", VFAT ? correctVFATPath( path ) : path );
	for ( QStringList::Iterator it = tokens.begin(); it != tokens.end(); ++it )
	{
		parcialPath.setPath( parcialPath.path() + "/" + *it );
		if ( !QFileInfo( parcialPath.path() ).exists() )
		{
			if ( !QDir().mkdir( parcialPath.path() ) )
				return false;
			if ( createdDirsList )
				createdDirsList->prepend( parcialPath.path() );
		}
	}

	return true;
}

bool System::copy( const QString& srcPath, const QString& dstPath, bool srcVFAT, bool dstVFAT )
{
	QFile srcFile( srcVFAT ? correctVFATPath( srcPath ) : srcPath );
	if ( !srcFile.open( IO_ReadOnly ) )
	{
		LOG( "copy: couldn't open src", SYSTEM );
		return false;
	}

	QFile dstFile( dstVFAT ? correctVFATPath( dstPath ) : dstPath );
	if ( !dstFile.open( IO_WriteOnly ) )
	{
		LOG( "copy: couldn't open dst", SYSTEM );
		return false;
	}

	char* data = new char[1000000];
	int read = srcFile.readBlock( data, 1000000 );
	int write = 0;
	while ( read != -1 && read != 0 && write != -1 )
	{
		write = dstFile.writeBlock( data, read );
		read = srcFile.readBlock( data, 1000000 );
	}

	srcFile.close();
	dstFile.close();

	if ( read == -1 || write == -1 )
	{
		LOG( "copy: error reading data from src", (read == -1) ? SYSTEM : NEVER );
		LOG( "copy: error writing data to dst", (write == -1) ? SYSTEM : NEVER );
		return false;
	}
	else
	{
		LOG( "copy: finished ok", SYSTEM );
		return true;
	}
}

bool System::move( const QString& srcPath, const QString& dstPath, bool srcVFAT, bool dstVFAT )
{
	QString src = srcVFAT ? correctVFATPath( srcPath ) : srcPath;
	QString dst = dstVFAT ? correctVFATPath( dstPath ) : dstPath;

	if ( !System::copy( src, dst, false, false ) )
	{
		LOG( "move: could not copy src to dst", SYSTEM );
		return false;
	}
	else
	{
		LOG( "move: error removing src", SYSTEM );
		return QFile::remove( src );
	}
}

bool System::remove( const QString& path, bool VFAT )
{
	return QFile::remove( VFAT ? correctVFATPath( path ) : path );
}

bool System::isReadable( const QString& path )
{
	QDir absPath( path );
	absPath.convertToAbs();
	QFileInfo fileInfo( absPath.path() );
	return fileInfo.isFile() && fileInfo.isReadable();
}

bool System::isWritable( const QString& path )
{
	QDir absPath( path );
	absPath.convertToAbs();
	QFileInfo fileInfo( absPath.path() );
	return fileInfo.isFile() && fileInfo.isWritable();
}

QString System::user()
{
#ifdef Q_WS_WIN
	// TODO Windows code not tested!
	// Taken from http://lists.trolltech.com/qt-interest/2002-12/thread00130-0.html
	#ifdef UNICODE
	if ( qWinVersion() & Qt::WV_NT_based )
	{
		TCHAR winUserName[UNLEN + 1]; // UNLEN is defined in LMCONS.H
		DWORD winUserNameSize = sizeof( winUserName );
		GetUserName( winUserName, &winUserNameSize );
		return qt_winQString( winUserName );
	}
	else
	#endif
	{
		char winUserName[UNLEN + 1]; // UNLEN is defined in LMCONS.H
		DWORD winUserNameSize = sizeof( winUserName );
		GetUserNameA( winUserName, &winUserNameSize );
		return QString::fromLocal8Bit( winUserName );
	}
#else
	char* userName = getlogin();
	if ( userName )
		return QString( userName );
	else
		if ( (userName = getenv( "USER" )) )
			return QString( userName );
		else
			return QString::null;
#endif
}


QString System::tempDir()
{
#ifdef Q_WS_WIN
	// TODO Windows code not tested!
	char* aux;
	if ( aux = ::getenv( "TEMP" ) )
		return aux;
	else if ( aux = ::getenv( "TMP" ) )
		return aux;
	else if ( aux = ::getenv( "SystemRoot" ) )
		return QString( aux ) + QDir::separator() + "temp";
	else if ( aux = ::getenv( "windir" ) )
		return QString( aux ) + QDir::separator() + "temp";
	else if ( qWinVersion() & Qt::WV_NT_based )
		return "C:\\WINNT\\temp";
	else
		return "C:\\WINDOWS\\temp";
#else
	return "/tmp";
#endif
}

#ifdef LOG_FILE
void clearLogFile()
{
	QFile file( LOG_FILE );
	if ( file.open( IO_WriteOnly|IO_Truncate ) )
		file.close();
}

void writeToLogFile( const QString& message )
{
	// The mutex is here because the function is used for
	// logging into the same file by more than one thread
	static QMutex writeToFileMutex;

	writeToFileMutex.lock();

	QFile file( LOG_FILE );
	if ( file.open( IO_WriteOnly|IO_Append ) )
	{
 		QTextStream textStream( &file );
		textStream << QThread::currentThread() << ": " << message << "\n";
		file.close();
	}

	writeToFileMutex.unlock();
}
#endif
