/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "transkodelogger.h"
#include <commondefs.h>

#include <qthread.h>
#include <qtextstream.h>
#include <qfileinfo.h>
#include <qfile.h>

TransKodeLogger::TransKodeLogger( const QString& path, const QString& prefix, bool truncate ):
	m_path( path ),
	m_file( QString::null ),
	m_prefix( prefix ),
	m_mutex( false )
{
	setFile( path, truncate, false );
}


TransKodeLogger::~TransKodeLogger()
{
	m_mutex.lock();

	if ( ! m_path.isEmpty() )
		m_file.close();

	m_mutex.unlock();
}

QString TransKodeLogger::file()
{
	m_mutex.lock();

	QString ret = m_path.isEmpty() ? QString::null : String::deepCopy( m_path );

	m_mutex.unlock();

	return ret;
}

void TransKodeLogger::setFile( const QString& path, bool truncate, bool force )
{
	m_mutex.lock();

	if ( force || path != m_path )
	{
		if ( ! m_path.isEmpty() )
			m_file.close();
	
		m_path = path;
		m_file.setName( m_path );
	
		if ( ! m_path.isEmpty() )
		{
			QFileInfo fileInfo( m_path );
			if ( !fileInfo.exists() || (fileInfo.exists() && fileInfo.isFile()) )
				if ( ! m_file.open( truncate ? IO_WriteOnly|IO_Truncate : IO_WriteOnly|IO_Append ) )
					m_path = QString::null;
		}
	}

	m_mutex.unlock();
}

void TransKodeLogger::setMsgPrefix( const QString& prefix )
{
	m_mutex.lock();

	m_prefix = prefix;

	m_mutex.unlock();
}

void TransKodeLogger::truncate()
{
	m_mutex.lock();

	if ( ! m_path.isEmpty() )
	{
		m_file.close();
		if ( ! m_file.open( IO_WriteOnly|IO_Truncate ) )
			m_path = QString::null;
	}

	m_mutex.unlock();
}

void TransKodeLogger::log( const QString message )
{
	m_mutex.lock();

	static QCString newLine( "\n" );

	if ( ! m_path.isEmpty() )
	{
		/*m_file.writeBlock( m_prefix.local8Bit() );
		m_file.writeBlock( message.local8Bit() );
		m_file.writeBlock( newLine );*/
		QTextStream textStream( &m_file );
		textStream << m_prefix << message << "\n";
		m_file.flush();
	}

	m_mutex.unlock();
}
