#ifndef _BONKENCODERPLUGIN_H_
#define _BONKENCODERPLUGIN_H_

/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <transkodeencoderplugin.h>
#include "bonkencoderdialog.h"
#include "bonkprogressparser.h"

class BonkEncoderPlugin : public TransKodeEncoderPlugin
{
	friend class TransKodePluginsManager;

	public:

		virtual bool configurable() const
		{
			return true;
		}

		virtual void setupArgs( QProcessWrapper& proc, const QString& src, const QString& dst, const QString& switches, const QString metadata[]) const
		{
			proc.clearArguments();
			bool pMode = proc.getShellParsingMode();
			proc.setShellParsingMode( false );
			proc << programPath() << "encode" << "-o" << dst;
			if ( ! metadata[Tag::artist].isEmpty() )	{ proc << "-a" << metadata[Tag::artist]; }
			if ( ! metadata[Tag::title].isEmpty() )		{ proc << "-t" << metadata[Tag::title]; }
			if ( ! metadata[Tag::comment].isEmpty() )	{ proc << "-c" << metadata[Tag::comment]; }
			proc.setShellParsingMode( true );
			proc << switches;
			proc.setShellParsingMode( false );
			proc << src;
			proc.setShellParsingMode( pMode );
		}

		virtual ProgressParser* newProgressParser() const
		{
			return new BonkProgressParser();
		}

		virtual TagLib::File* newTagLibFile( const char*, bool, int  ) const
		{
			return 0;
		}

	protected:

		virtual EncoderDialog* _newEncoderDialog( QWidget* parent=0, const char* name=0 ) const
		{
			return new BonkEncoderDialog( parent, name );
		}

		BonkEncoderPlugin():TransKodeEncoderPlugin( "Bonk", QString::null, "bonk", QStringList::split( ":", "bonk:*" ) ) {}

		~BonkEncoderPlugin() {}
};

#endif
 
