#ifndef _FAACENCODERPLUGIN_H_
#define _FAACENCODERPLUGIN_H_

/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <transkodeencoderplugin.h>
#include <processoutput.h>
#include "faacprogressparser.h"
#include "faacencoderdialog.h"

class FAACEncoderPlugin : public TransKodeEncoderPlugin
{
	friend class TransKodePluginsManager;

	public:

		virtual bool configurable() const
		{
			return true;
		}

		virtual void setupArgs( QProcessWrapper& proc, const QString& src, const QString& dst, const QString& switches, const QString metadata[] ) const
		{
			proc.clearArguments();
			bool pMode = proc.getShellParsingMode();
			proc.setShellParsingMode( false );
			proc << programPath();
			proc.setShellParsingMode( true );
		 	proc << switches;
			proc.setShellParsingMode( false );

			if ( getMetadataSupported() )
			{
//				TransKodePluginsManager::log( "[FAAC] metadata support found" );

				if ( ! metadata[Tag::artist].isEmpty() )	{ proc << "--artist" << metadata[Tag::artist]; }
				if ( ! metadata[Tag::year].isEmpty() )		{ proc << "--year" << metadata[Tag::year]; }
				if ( ! metadata[Tag::album].isEmpty() )		{ proc << "--album" << metadata[Tag::album]; }
				if ( ! metadata[Tag::track].isEmpty() )		{ proc << "--track" << metadata[Tag::track]; }
				if ( ! metadata[Tag::title].isEmpty() )		{ proc << "--title" << metadata[Tag::title]; }
				if ( ! metadata[Tag::comment].isEmpty() )	{ proc << "--comment" << metadata[Tag::comment]; }
				if ( ! metadata[Tag::genre].isEmpty() )		{ proc << "--genre" << metadata[Tag::genre]; }
			}
// 			else
// 				TransKodePluginsManager::log( "[FAAC] metadata support not found" );

			proc << "-o" << dst << src;
			proc.setShellParsingMode( pMode );
		}

		virtual ProgressParser* newProgressParser() const
		{
			return new FAACProgressParser();
		}

		virtual TagLib::File* newTagLibFile( const char*, bool, int ) const
		{
			return 0;
		}

	protected:

		virtual void setProgramPath( const QString& path )
		{
			if ( path != programPath() )
			{
				TransKodeEncoderPlugin::setProgramPath( path );
				checkMetadataSupport();
			}
		}

		virtual EncoderDialog* _newEncoderDialog( QWidget* parent=0, const char* name=0 ) const
		{
			return new FAACEncoderDialog( parent, name );
		}

		FAACEncoderPlugin():
			TransKodeEncoderPlugin( "FAAC", QString::null, "faac", QStringList::split( ":", "mp4:m4a:aac:*" ) )
		{
			checkMetadataSupport();
		}

		~FAACEncoderPlugin() {}


		void checkMetadataSupport()
		{
			//QProcessWrapper process( 0, QProcess::Stdout|QProcess::Stderr|QProcess::DupStderr );
			QProcessWrapper process( 0, QProcess::Stderr );
			process <<  programPath() << "--long-help";

			OutputGrabber* outputGrabber = new OutputGrabber( &process );

			if ( process.start() )
			{
				process.waitProcess( 1000 );
				setMetadataSupported( outputGrabber->getOutput().contains( "--artist X" ) );
			}
			else
				setMetadataSupported( false );

			LOG( QString("[FAAC] metadata support ") + (getMetadataSupported() ? "found" : "not found"), ALWAYS );

			delete outputGrabber;
		}

		void setMetadataSupported( bool value )
		{
			m_mutex.lock();

			m_mdSupported = value;

			m_mutex.unlock();
		}

		bool getMetadataSupported() const
		{
			m_mutex.lock();

			bool ret = m_mdSupported;

			m_mutex.unlock();

			return ret;
		}

		bool m_mdSupported;
};

#endif
 
