#ifndef _LAMEENCODERPLUGIN_H_
#define _LAMEENCODERPLUGIN_H_

/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <transkodeencoderplugin.h>
#include "lameprogressparser.h"
#include "lameencoderdialog.h"

#include <audioproperties.h>
#include <mpegfile.h>

class LAMEEncoderPlugin : public TransKodeEncoderPlugin
{
	friend class TransKodePluginsManager;

	public:

		virtual bool configurable() const
		{
			return true;
		}

		virtual void setupArgs( QProcessWrapper& proc, const QString& src, const QString& dst, const QString& switches, const QString metadata[]  ) const
		{
			proc.clearArguments();
			bool pMode = proc.getShellParsingMode();
			proc.setShellParsingMode( false );
			proc << programPath() << "--nohist";
			proc.setShellParsingMode( true );
			proc << switches;
			proc.setShellParsingMode( false );
			proc << "--ignore-tag-errors" << "--add-id3v2";
			if ( ! metadata[Tag::artist].isEmpty() )	{ proc << "--ta" << metadata[Tag::artist]; }
			if ( ! metadata[Tag::year].isEmpty() )		{ proc << "--ty" << metadata[Tag::year]; }
			if ( ! metadata[Tag::album].isEmpty() )		{ proc << "--tl" << metadata[Tag::album]; }
			if ( ! metadata[Tag::track].isEmpty() )		{ proc << "--tn" << metadata[Tag::track]; }
			if ( ! metadata[Tag::title].isEmpty() )		{ proc << "--tt" << metadata[Tag::title]; }
			if ( ! metadata[Tag::comment].isEmpty() )	{ proc << "--tc" << metadata[Tag::comment]; }
			if ( ! metadata[Tag::genre].isEmpty() )		{ proc << "--tg" << metadata[Tag::genre]; }
			proc << src << dst;
			proc.setShellParsingMode( pMode );
		}

		virtual ProgressParser* newProgressParser() const
		{
			return new LAMEProgressParser( LAMEProgressParser::Encoder );
		}

		virtual TagLib::File* newTagLibFile( const char* fileName, bool readAudioProp, int audioPropStyle ) const
		{
			return new TagLib::MPEG::File( fileName, readAudioProp, (TagLib::AudioProperties::ReadStyle)audioPropStyle );
		}

	protected:

		virtual EncoderDialog* _newEncoderDialog( QWidget* parent=0, const char* name=0 ) const
		{
			return new LAMEEncoderDialog( parent, name );
		}

		LAMEEncoderPlugin():TransKodeEncoderPlugin( "LAME", QString::null, "lame", QStringList::split( ":", "mp3:*" ) ) {}

		~LAMEEncoderPlugin() {}
};

#endif
 
