#ifndef _QPROCESSWRAPPER_H_
#define _QPROCESSWRAPPER_H_

/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <qprocess.h>
#include <qwaitcondition.h>
#include <qstring.h>
#include <qstringlist.h>

class QProcessWrapper : public QProcess
{
	Q_OBJECT
	public:
		typedef enum {
			POSIX_SIGABRT = 0, POSIX_SIGALRM, POSIX_SIGFPE, POSIX_SIGHUP,
			POSIX_SIGILL, POSIX_SIGINT, POSIX_SIGKILL, POSIX_SIGPIPE,
			POSIX_SIGQUIT, POSIX_SIGSEGV, POSIX_SIGTERM, POSIX_SIGUSR1,
			POSIX_SIGUSR2, POSIX_SIGCHLD, POSIX_SIGCONT, POSIX_SIGSTOP,
			POSIX_SIGTSTP, POSIX_SIGTTIN, POSIX_SIGTTOU
		} Signal;

		QProcessWrapper( int priority = 0, int comm = 0, QObject* parent = 0, const char* name = 0 );
		QProcessWrapper( const QString& arg0, int priority = 0, int comm = 0, QObject* parent = 0, const char* name = 0 );
		QProcessWrapper( const QStringList& args, int priority = 0, int comm = 0, QObject* parent = 0, const char* name = 0 );
		virtual ~QProcessWrapper();

		virtual bool start( QStringList* env = 0 );

		void terminate( unsigned msecs );

		bool waitProcess( unsigned msecs = (unsigned)-1 );

		void setShellParsingMode( bool value );
		bool getShellParsingMode();

		virtual void setArguments( const QStringList& args );
		virtual void addArgument( const QString& arg );
		virtual void addArguments( const QStringList& args );

		QProcessWrapper& operator<<( const QString& arg );
		QProcessWrapper& operator<<( const char* arg );
		QProcessWrapper& operator<<( const QByteArray& arg );
		QProcessWrapper& operator<<( const QStringList& args );

		static QStringList parseShellArgs( const QString& args );

		int priority() const;
		bool setPriority( int value );

		bool paused() const;
		bool pause();
		bool resume();

		bool signal( Signal sig );

	protected:
		int m_priority;
		bool m_paused;
		bool m_shellParsing;
		QWaitCondition m_waitCondition;
};

#endif

