/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "qdecspinbox.h"

QDecSpinBox::QDecSpinBox( QWidget *parent, const char *name ):QSpinBox( parent, name )
{
	val = new QDecValidator( this );
	setValidator( val );

	setDecPositions( 2 );
}

int positivePower( int value, unsigned power )
{
	int res = 1;
	for ( unsigned count = 0; count < power; ++count )
		res *= value;
	return res;
}

void QDecSpinBox::setDecPositions( int decPositions )
{
	if ( decPositions > 0 )
	{
		decPos = decPositions;
		decConst = positivePower( 10, decPositions );
		val->setBottom( minValue()/decConst );
		val->setTop( maxValue()/decConst );
		//val->setDecimals( decPos );
		setValue( value() ); // Update control representation
	}
}

void QDecSpinBox::setDecValue( double v )
{
//	if ( v != decValue() )
		setValue( (int)(v*decConst+0.5) );
}

double QDecSpinBox::decValue() const
{
	return ((double)value())/decConst;
}

int QDecSpinBox::decPositions() const
{
	return decPos;
}

QString QDecSpinBox::mapValueToText( int v )
{
	return QString().sprintf( "%." + QString::number(decPos) + "f", ((double)v)/decConst );
}

int QDecSpinBox::mapTextToValue( bool* ok )
{
	return (int)(cleanText().toFloat( ok )*decConst+0.5);
}
