#ifndef _TRANSKODEFILEJOBEVENTS_H_
#define _TRANSKODEFILEJOBEVENTS_H_

/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <commondefs.h>

#include <qevent.h>
#include <qstring.h>
#include <qstringlist.h>

#define PROFILEERRORSETEVENT	(QEvent::User + 1)
#define PROFILEERRORUNSETEVENT	(QEvent::User + 2)

#define DSTPATHCHANGEDEVENT		(QEvent::User + 3)

#define ENQUEUEDEVENT			(QEvent::User + 4)
#define POSCHANGEDEVENT			(QEvent::User + 5)
#define CANCELLEDEVENT			(QEvent::User + 6)

#define STARTEDEVENT			(QEvent::User + 7)
#define FINISHEDEVENT			(QEvent::User + 8)
#define ERROREVENT				(QEvent::User + 9)
#define PROGRESSEVENT			(QEvent::User + 10)

class ProfileErrorSetEvent : public QCustomEvent
{
	public:
		ProfileErrorSetEvent( const QString& srcPath ):
			QCustomEvent( PROFILEERRORSETEVENT ),
			m_srcPath( String::deepCopy( srcPath ) ) {};

		QString m_srcPath;
};

class ProfileErrorUnsetEvent : public QCustomEvent
{
	public:
		ProfileErrorUnsetEvent( const QString& srcPath ):
			QCustomEvent( PROFILEERRORUNSETEVENT ),
			m_srcPath( String::deepCopy( srcPath ) ) {};

		QString m_srcPath;
};

class DstPathChangedEvent : public QCustomEvent
{
	public:
		DstPathChangedEvent( const QString dstPath ):
			QCustomEvent( DSTPATHCHANGEDEVENT ),
			m_dstPath( dstPath ) {};

		QString m_dstPath;
};


class EnqueuedEvent : public QCustomEvent
{
	public:
		EnqueuedEvent( int queuePos ):
			QCustomEvent( ENQUEUEDEVENT ),
			m_queuePos( queuePos ) {};

		int m_queuePos;
};

class PosChangedEvent : public QCustomEvent
{
	public:
		PosChangedEvent( int queuePos ):
			QCustomEvent( POSCHANGEDEVENT ),
			m_queuePos( queuePos ) {};

		int m_queuePos;
};

class CancelledEvent : public QCustomEvent
{
	public:
		CancelledEvent():
			QCustomEvent( CANCELLEDEVENT ) {};
};


class StartedEvent : public QCustomEvent
{
	public:
		StartedEvent( const QString& srcPath, const QString& dstPath, int jobType ):
			QCustomEvent( STARTEDEVENT ),
			m_jobType( jobType ),
			m_srcPath( String::deepCopy( srcPath ) ),
			m_dstPath( String::deepCopy( dstPath ) ) {};

		int m_jobType;
		QString m_srcPath;
		QString m_dstPath;
};

class FinishedEvent : public QCustomEvent
{
	public:
		FinishedEvent( const QString& srcPath, const QString& dstPath, int jobType ):
			QCustomEvent( FINISHEDEVENT ),
			m_jobType( jobType ),
			m_srcPath( String::deepCopy( srcPath ) ),
			m_dstPath( String::deepCopy( dstPath ) ) {};

		int m_jobType;
		QString m_srcPath;
		QString m_dstPath;
};

class ErrorEvent : public QCustomEvent
{
	public:
		ErrorEvent( const QString& srcPath, const QString& errorMsg, int jobType ):
			QCustomEvent( ERROREVENT ),
			m_jobType( jobType ),
			m_srcPath( srcPath ),
			m_errorMsg( errorMsg ) {};

		int m_jobType;
		QString m_srcPath;
		QString m_errorMsg;
};

class ProgressEvent : public QCustomEvent
{
	public:
		ProgressEvent( int progress, int jobType ):
			QCustomEvent( PROGRESSEVENT ),
			m_jobType( jobType ),
			m_progress( progress ) {};

		int m_jobType;
		int m_progress;
};


#endif
