#ifndef _TRANSKODEFILEWORKEREVENTS_H_
#define _TRANSKODEFILEWORKEREVENTS_H_

/***************************************************************************
 *   Copyright (C) 2005 by Sergio Pistone                                  *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <qevent.h>
#include <qstring.h>
#include <qstringlist.h>

#define ROUNDSTARTEDEVENT		(QEvent::User + 1)
#define ROUNDFINISHEDEVENT		(QEvent::User + 2)
#define ROUNDSHORTSTATSEVENT	(QEvent::User + 3)
#define ROUNDLONGSTATSEVENT		(QEvent::User + 4)

#define PROCESSPAUSEDEVENT		(QEvent::User + 5)
#define PROCESSRESUMEDEVENT		(QEvent::User + 6)

class RoundStartedEvent : public QCustomEvent
{
	public:
		RoundStartedEvent():
			QCustomEvent( ROUNDSTARTEDEVENT ) {};
};

class RoundFinishedEvent : public QCustomEvent
{
	public:
		RoundFinishedEvent():
			QCustomEvent( ROUNDFINISHEDEVENT ) {};
};

class RoundShortStatsEvent : public QCustomEvent
{
	public:
		RoundShortStatsEvent(
			int transcodeE, int transcodeF,
			int copyE, int copyF,
			int jobsC, int profileE ):

			QCustomEvent( ROUNDSHORTSTATSEVENT ),
			m_transcodeE( transcodeE ), m_transcodeF( transcodeF ),
			m_copyE( copyE ), m_copyF( copyF ),
			m_jobsC( jobsC ), m_profileE( profileE ) {};

		int m_transcodeE; int m_transcodeF;
		int m_copyE; int m_copyF;
		int m_jobsC; int m_profileE;
};

class RoundLongStatsEvent : public QCustomEvent
{
	public:
		RoundLongStatsEvent(
			int transcodeE, const QStringList& tSrcFiles, const QStringList& tDstFiles,
			int copyE, const QStringList& cSrcFiles, const QStringList& cDstFiles,
			int jobsC, int profileE ):

			QCustomEvent( ROUNDLONGSTATSEVENT ),
			m_transcodeE( transcodeE ), m_tSrcFiles( tSrcFiles ), m_tDstFiles( tDstFiles ),
			m_copyE( copyE ), m_cSrcFiles( cSrcFiles ), m_cDstFiles( cDstFiles ),
			m_jobsC( jobsC ), m_profileE( profileE ) {};

		int m_transcodeE; const QStringList& m_tSrcFiles; const QStringList& m_tDstFiles;
		int m_copyE; const QStringList& m_cSrcFiles; const QStringList& m_cDstFiles;
		int m_jobsC; int m_profileE;
};


class ProcessPausedEvent : public QCustomEvent
{
	public:
		ProcessPausedEvent():
			QCustomEvent( PROCESSPAUSEDEVENT ) {};
};

class ProcessResumedEvent : public QCustomEvent
{
	public:
		ProcessResumedEvent():
			QCustomEvent( PROCESSRESUMEDEVENT ) {};
};

#endif
